/*****************************************************************************
 * This file is part of the BlueDevil project                                *
 *                                                                           *
 * Copyright (C) 2011 Rafael Fernández López <ereslibre@kde.org>             *
 * Copyright (C) 2011 UFO Coders <info@ufocoders.com>                        *
 *                                                                           *
 * This library is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU Library General Public               *
 * License as published by the Free Software Foundation; either              *
 * version 2 of the License, or (at your option) any later version.          *
 *                                                                           *
 * This library is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         *
 * Library General Public License for more details.                          *
 *                                                                           *
 * You should have received a copy of the GNU Library General Public License *
 * along with this library; see the file COPYING.LIB.  If not, write to      *
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,      *
 * Boston, MA 02110-1301, USA.                                               *
 *****************************************************************************/

#ifndef ADAPTERTEST_H
#define ADAPTERTEST_H

#include <QtCore/QObject>
#include <QtCore/QThread>

namespace BlueDevil {
    class Adapter;
    class Device;
}

using namespace BlueDevil;

class AdapterTest
    : public QThread
{
    Q_OBJECT

public:
    AdapterTest(QObject *parent = 0);
    virtual ~AdapterTest();

private Q_SLOTS:
    void adapterAdded(Adapter *adapter);
    void adapterRemoved(Adapter *adapter);
    void defaultAdapterChanged(Adapter *adapter);
    void usableAdapterChanged(Adapter *adapter);
    void allAdaptersRemoved();

protected:
    virtual void run();
};

#endif // ADAPTERTEST_H
